@@ -27,6 +27,9 @@
 #include <KLocalizedString>
 
 FavoritesModel::FavoritesModel(QObject *parent) : AbstractModel(parent)
+, m_enabled(true)
+, m_maxFavorites(-1)
+, m_dropPlaceholderIndex(-1)
 {
 }
 
@@ -42,17 +45,32 @@ QString FavoritesModel::description() const
 
 QVariant FavoritesModel::data(const QModelIndex& index, int role) const
 {
-
-    if (!index.isValid() || index.row() >= m_entryList.count()) {
+    if (!index.isValid() || index.row() >= rowCount()) {
         return QVariant();
     }
 
-    const AbstractEntry *entry = m_entryList.at(index.row());
+    if (index.row() == m_dropPlaceholderIndex) {
+        if (role == Kicker::IsDropPlaceholderRole) {
+            return true;
+        } else {
+            return QVariant();
+        }
+    }
+
+    int mappedIndex = index.row();
+
+    if (m_dropPlaceholderIndex != -1 && mappedIndex > m_dropPlaceholderIndex) {
+        --mappedIndex;
+    }
+
+    const AbstractEntry *entry = m_entryList.at(mappedIndex);
 
     if (role == Qt::DisplayRole) {
         return entry->name();
     } else if (role == Qt::DecorationRole) {
         return entry->icon();
+    } else if (role == Kicker::DescriptionRole) {
+        return entry->description();
     } else if (role == Kicker::FavoriteIdRole) {
         return entry->id();
     } else if (role == Kicker::UrlRole) {
@@ -68,7 +86,7 @@ QVariant FavoritesModel::data(const QModelIndex& index, int role) const
 
 int FavoritesModel::rowCount(const QModelIndex& parent) const
 {
-    return parent.isValid() ? 0 : m_entryList.count();
+    return parent.isValid() ? 0 : m_entryList.count() + (m_dropPlaceholderIndex != -1 ? 1 : 0);
 }
 
 bool FavoritesModel::trigger(int row, const QString &actionId, const QVariant &argument)
@@ -80,6 +98,20 @@ bool FavoritesModel::trigger(int row, const QString &actionId, const QVariant &a
     return m_entryList.at(row)->run(actionId, argument);
 }
 
+bool FavoritesModel::enabled() const
+{
+    return m_enabled;
+}
+
+void FavoritesModel::setEnabled(bool enable)
+{
+    if (m_enabled != enable) {
+        m_enabled = enable;
+
+        emit enabledChanged();
+    }
+}
+
 QStringList FavoritesModel::favorites() const
 {
     return m_favorites;
@@ -96,14 +128,37 @@ void FavoritesModel::setFavorites(const QStringList& favorites)
     }
 }
 
+int FavoritesModel::maxFavorites() const
+{
+    return m_maxFavorites;
+}
+
+void FavoritesModel::setMaxFavorites(int max)
+{
+    if (m_maxFavorites != max)
+    {
+        m_maxFavorites = max;
+
+        if (m_maxFavorites != -1 && m_favorites.count() > m_maxFavorites) {
+            refresh();
+        }
+
+        emit maxFavoritesChanged();
+    }
+}
+
 bool FavoritesModel::isFavorite(const QString &id) const
 {
     return m_favorites.contains(id);
 }
 
-void FavoritesModel::addFavorite(const QString &id)
+void FavoritesModel::addFavorite(const QString &id, int index)
 {
-    if (id.isEmpty()) {
+    if (!m_enabled || id.isEmpty()) {
+        return;
+    }
+
+    if (m_maxFavorites != -1 && m_favorites.count() == m_maxFavorites) {
         return;
     }
 
@@ -114,10 +169,14 @@ void FavoritesModel::addFavorite(const QString &id)
         return;
     }
 
-    beginInsertRows(QModelIndex(), m_entryList.count(), m_entryList.count());
+    setDropPlaceholderIndex(-1);
 
-    m_entryList << entry;
-    m_favorites << entry->id();
+    int insertIndex = (index != -1) ? index : m_entryList.count();
+
+    beginInsertRows(QModelIndex(), insertIndex, insertIndex);
+
+    m_entryList.insert(insertIndex, entry);
+    m_favorites.insert(insertIndex, entry->id());
 
     endInsertRows();
 
@@ -127,9 +186,15 @@ void FavoritesModel::addFavorite(const QString &id)
 
 void FavoritesModel::removeFavorite(const QString &id)
 {
+    if (!m_enabled || id.isEmpty()) {
+        return;
+    }
+
     int index = m_favorites.indexOf(id);
 
     if (index != -1) {
+        setDropPlaceholderIndex(-1);
+
         beginRemoveRows(QModelIndex(), index, index);
 
         delete m_entryList[index];
@@ -153,6 +218,8 @@ void FavoritesModel::moveRow(int from, int to)
         return;
     }
 
+    setDropPlaceholderIndex(-1);
+
     int modelTo = to + (to > from ? 1 : 0);
 
     bool ok = beginMoveRows(QModelIndex(), from, from, QModelIndex(), modelTo);
@@ -167,6 +234,42 @@ void FavoritesModel::moveRow(int from, int to)
     }
 }
 
+int FavoritesModel::dropPlaceholderIndex() const
+{
+    return m_dropPlaceholderIndex;
+}
+
+void FavoritesModel::setDropPlaceholderIndex(int index)
+{
+    if (index == -1 && m_dropPlaceholderIndex != -1) {
+        beginRemoveRows(QModelIndex(), m_dropPlaceholderIndex, m_dropPlaceholderIndex);
+
+        m_dropPlaceholderIndex = index;
+
+        endRemoveRows();
+
+        emit countChanged();
+    } else if (index != -1 && m_dropPlaceholderIndex == -1) {
+        beginInsertRows(QModelIndex(), index, index);
+
+        m_dropPlaceholderIndex = index;
+
+        endInsertRows();
+
+        emit countChanged();
+    } else if (m_dropPlaceholderIndex != index) {
+        int modelTo = index + (index > m_dropPlaceholderIndex ? 1 : 0);
+
+        bool ok = beginMoveRows(QModelIndex(), m_dropPlaceholderIndex, m_dropPlaceholderIndex, QModelIndex(), modelTo);
+
+        if (ok) {
+            m_dropPlaceholderIndex = index;
+
+            endMoveRows();
+        }
+    }
+}
+
 AbstractModel *FavoritesModel::favoritesModel()
 {
     return this;
@@ -176,6 +279,8 @@ void FavoritesModel::refresh()
 {
     beginResetModel();
 
+    setDropPlaceholderIndex(-1);
+
     int oldCount = m_entryList.count();
 
     qDeleteAll(m_entryList);
@@ -189,6 +294,10 @@ void FavoritesModel::refresh()
         if (entry && entry->isValid()) {
             m_entryList << entry;
             newFavorites << entry->id();
+
+            if (m_maxFavorites != -1 && newFavorites.count() == m_maxFavorites) {
+                break;
+            }
         } else if (entry) {
             delete entry;
         }
@@ -214,15 +323,10 @@ AbstractEntry *FavoritesModel::favoriteFromId(const QString &id)
         return new AppEntry(this, id);
     } else if (s == QStringLiteral("ktp")) {
         return new ContactEntry(this, id);
+    } else if (url.isValid() && !url.scheme().isEmpty()) {
+        return new FileEntry(this, url);
     } else {
-        AbstractEntry *entry = new SystemEntry(this, id);
-
-        if (!entry->isValid()) {
-            delete entry;
-            return new FileEntry(this, url);
-        }
-
-        return entry;
+        return new SystemEntry(this, id);
     }
 
     return nullptr;
